import os
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from shondesh.channels.base import Channel, logger
from shondesh.formatters.base_formatter import Formatter
from shondesh.formatters.dict_table_formatter import DictTableFormatter


class Email(Channel):
    """Email alert notifier implementation"""

    def __init__(
        self, config: dict = None, formatter: Formatter = DictTableFormatter()
    ):
        super().__init__(config=config, formatter=formatter)
        self.config = config
        self.formatter = formatter

    async def send(self, data: dict) -> bool:
        try:
            msg = MIMEMultipart()
            msg["From"] = self.config["from_address"]
            msg["To"] = ", ".join(self.config["recipients"])
            msg["Subject"] = self.config["subject_template"]

            body = self.formatter.format(data)

            msg.attach(MIMEText(body, "plain"))

            password = self.config["password"]
            if password.startswith("${") and password.endswith("}"):
                env_var = password[2:-1]
                password = os.getenv(env_var, password)

            server = smtplib.SMTP(self.config["smtp_server"], self.config["smtp_port"])
            server.starttls()
            server.login(self.config["username"], password)
            text = msg.as_string()
            server.sendmail(
                self.config["from_address"], self.config["recipients"], text
            )
            server.quit()

            return True
        except Exception as e:
            logger.error(f"Failed to send email alert: {e}")
            return False



from abc import ABC, abstractmethod
from typing import Dict, Any
import logging

from shondesh.formatters.base_formatter import Formatter

logger = logging.getLogger(__name__)


class Channel(ABC):
    """Abstract base class for channels"""

    def __init__(self, config: Dict, formatter: Formatter = None):
        self.config = config
        self.formatter = formatter

    @abstractmethod
    async def send(self, data: Any) -> bool:
        """Send alert for violation"""
        pass



from shondesh.formatters.base_formatter import Formatter


class DictTableFormatter(Formatter):
    def format(self, data: dict) -> str:
        if not data:
            return ""
        key_width = max(len(str(k)) for k in data.keys())
        val_width = max(len(str(v)) for v in data.values())
        lines = [
            f"{'Key'.ljust(key_width)} | {'Value'.ljust(val_width)}",
            f"{'-' * key_width}-+-{'-' * val_width}",
        ]
        for k, v in data.items():
            lines.append(f"{str(k).ljust(key_width)} | {str(v).ljust(val_width)}")
        return "\n".join(lines)



class SlackMessageFormatter:

    def format(self, message: dict) -> list:
        """
        Format the message for Slack.
        """
        if not message:
            return []

        # Escape special characters for Slack

        attachment_fields = []
        # Format the message with a code block
        for key, value in message.items():
            if isinstance(value, str):
                value = (
                    value.replace("`", "\\`").replace("*", "\\*").replace("_", "\\_")
                )
            attachment_fields.append({"title": key, "value": str(value), "short": True})

        return attachment_fields



from shondesh.channels.base import Channel, logger
from shondesh.formatters.base_formatter import Formatter
from shondesh.formatters.dict_table_formatter import DictTableFormatter


class Telegram(Channel):
    """Telegram alert channel implementation"""

    def __init__(self, config: dict, formatter: Formatter = DictTableFormatter()):
        super().__init__(config=config, formatter=formatter)
        self.config = config
        self.formatter = formatter

        if not self.config.get("webhook_url"):
            raise ValueError("Telegram webhook URL is required in the configuration.")
        if not self.config.get("chat_id"):
            raise ValueError("Telegram chat ID is required in the configuration.")

    async def send(self, data: dict) -> bool:
        import aiohttp

        try:
            payload = {
                "chat_id": self.config["chat_id"],
                "text": self.formatter.format(data),
                "parse_mode": "Markdown",
            }

            async with aiohttp.ClientSession() as session:
                async with session.post(
                    self.config["webhook_url"], json=payload
                ) as response:
                    return response.status == 200
        except Exception as e:
            logger.error(f"Failed to send Telegram alert: {e}")
            return False



import asyncio
import os

from shondesh.channels.base import Channel, logger
from shondesh.formatters.base_formatter import Formatter
from shondesh.formatters.dict_table_formatter import DictTableFormatter


class Webhook(Channel):
    """Webhook alert notifier implementation"""

    def __init__(self, config: dict, formatter: Formatter = DictTableFormatter()):
        super().__init__(config=config, formatter=formatter)
        self.config = config
        self.formatter = formatter

        if not self.config.get("url"):
            raise ValueError("Webhook URL is required in the configuration.")
        if not self.config.get("method"):
            self.config["method"] = "POST"

    async def send(self, data: dict) -> bool:
        import aiohttp

        try:
            headers = self.config.get("headers", {})
            # Replace environment variables in headers
            for key, value in headers.items():
                if (
                    isinstance(value, str)
                    and value.startswith("${")
                    and value.endswith("}")
                ):
                    env_var = value[2:-1]
                    headers[key] = os.getenv(env_var, value)

            payload = self.formatter.format(data)

            async with aiohttp.ClientSession() as session:
                for attempt in range(self.config.get("retry_count", 1)):
                    try:
                        response = await session.request(
                            self.config.get("method", "POST"),
                            self.config["url"],
                            json=payload,
                            headers=headers,
                        )
                        async with response:
                            if response.status < 400:
                                return True
                    except Exception as e:
                        if attempt == self.config.get("retry_count", 1) - 1:
                            raise e
                        await asyncio.sleep(1)

            return False
        except Exception as e:
            logger.error(f"Failed to send webhook alert: {e}")
            return False



class ChannelFactory:
    """
    Factory class to create channel instances based on the channel type.
    """

    @staticmethod
    def create_channel(channel_type, **kwargs):
        """
        Create a notifier instance based on the notifier type.

        :param channel_type: The type of notifier to create.
        :param kwargs: Additional parameters for the notifier.
        :return: An instance of the specified notifier.
        """
        from shondesh.channels import (
            Email,
            Telegram,
            Slack,
            Webhook,
        )

        channels = {
            "email": Email,
            "telegram": Telegram,
            "slack": Slack,
            "webhook": Webhook,
        }

        if channel_type not in channels:
            raise ValueError(f"Unsupported channel type: {channel_type}")

        return channels[channel_type](**kwargs)



from shondesh.channels.base import Channel, logger
from shondesh.formatters.base_formatter import Formatter
from shondesh.formatters.slack_message_formatter import SlackMessageFormatter
from shondesh.utils.constants import Severity


class Slack(Channel):
    """Slack alert channel implementation"""

    def __init__(self, config: dict, formatter: Formatter = SlackMessageFormatter()):
        super().__init__(config=config, formatter=formatter)

    async def send(self, data: dict) -> bool:
        import aiohttp

        try:
            severity = data.get("severity", Severity.INFO)
            payload = {
                "channel": self.config["channel"],
                "username": self.config["username"],
                "icon_emoji": self.config["icon_emoji"],
                "text": f"🚨 *{severity.value.upper()}*",
                "attachments": [
                    {
                        "color": (
                            "danger" if severity == Severity.CRITICAL else "warning"
                        ),
                        "fields": self.formatter.format(data) if self.formatter else [],
                    }
                ],
            }

            # Add mentions if configured
            if "mention_users" in self.config:
                payload["text"] = (
                    f"{' '.join(self.config['mention_users'])} {payload['text']}"
                )

            async with aiohttp.ClientSession() as session:
                async with session.post(
                    self.config["webhook_url"], json=payload
                ) as response:
                    return response.status == 200
        except Exception as e:
            logger.error(f"Failed to send Slack alert: {e}")
            return False
